/**
 * ====================================================================================
 * DemoOSMParser.java: Exercise OpenStreetMap SAX Parser ....
 *
 * Written by: Mark Austin                                                 October 2015
 * ====================================================================================
 */

package osm01;

import java.io.*;
import java.net.URL;
import java.util.*;

import java.io.*;
import java.net.URL;
import java.util.*;

import javax.xml.parsers.*;
import org.xml.sax.*;

import osm01.model.*;

public class DemoOSMParser {
   public static void main(String[] args){
      try {

      // File inputFile = new File("data/centralpark.osm");
      // File inputFile = new File("data/filtered-nyc-subway.osm");
      // File inputFile = new File("data/lowermanhattansubway.osm");
      // File inputFile = new File("data/newyorkareasubway.osm");
      // File inputFile = new File("data/new-york-ny.osm");
      // File inputFile = new File("data/new-york-ny-subway.osm");
      // File inputFile = new File("data/newyorktrainmap.osm");
      // File inputFile = new File("data/washingtonmetro.osm");
      // File inputFile = new File("data/washingtonmetromap.osm");
      // File inputFile = new File("data/nyc-coastline-simple01.osm");
      // File inputFile = new File("data/nyc-subway02.osm");
      // File inputFile = new File("data/wdc-river01.osm");

         File inputFile = new File("data/bwiairport.osm");

         System.out.printf("--- Read: %s ... \n", "data/bwiairport.osm" );
         System.out.printf("--- ============================================== \n");

         SAXParserFactory factory = SAXParserFactory.newInstance();
         SAXParser saxParser = factory.newSAXParser();

         OpenStreetMapSAXHandler osmhandler = new OpenStreetMapSAXHandler();
         saxParser.parse(inputFile, osmhandler);

         // Print names of OSM Node elements ... 

         SortedSet<String> names = osmhandler.getNodeNames();
         printNames("List of Node Names", names );

         // Print names of OSM Way elements ... 

         names = osmhandler.getWayNames();
         printNames("List of Way Names", names );

         // Print names of OSM Relation elements ... 

         names = osmhandler.getRelationNames();
         printNames("List of Relation Names", names );

         // Print size of OpenStreetMapNode TreeMap ....

         TreeMap<String, OpenStreetMapNode> nodes = osmhandler.getOpenStreetMapNodeTreeMap();
         TreeMap<String, Way>       ways = osmhandler.getOpenStreetMapWayTreeMap();
         TreeMap<String, Relation>  rels = osmhandler.getOpenStreetMapRelationTreeMap();

         System.out.printf("*** No nodes     = %d ... \n", nodes.size() );
         System.out.printf("*** No ways      = %d ... \n", ways.size() );
         System.out.printf("*** No relations = %d ... \n", rels.size() );

         printOSMNodeTypeMap( nodes );
         printOSMWayTypeMap( ways );
         printOSMRelationTypeMap( rels, ways );

         System.out.printf("--- ============================================== \n");
         System.out.printf("--- Finished!! \n");

      } catch (Exception e) {
         e.printStackTrace();
      }
   }

   // ===========================================================
   // Print node, way and relation names ...
   // ===========================================================

   public static void printNames(String title, SortedSet<String> nameSet ) {
      System.out.printf("*** %s ... \n", title );
      System.out.printf("*** ===============================\n");

      int i = 1;
      for (String name : nameSet) {
          System.out.printf("%4d: %s\n", i, name);
          i = i + 1;
      }
      System.out.println("*** ===============================");
   }

   // =======================================================================
   // Print contents of TreeMap<String, OpenStreetMapNode> osmNodeTypeMap ...
   // =======================================================================

   public static void printOSMNodeTypeMap( TreeMap<String, OpenStreetMapNode> osmNodeTypeMap ) {

      // Header for openstreetmap nodes ...

      System.out.println("***");
      System.out.println("*** Summary: TreeMap<String, OpenStreetMapNode> ... ");
      System.out.println("*** ==================================================== ... ");

      // Summary of categories of openstreetmap node ...

      Iterator<Map.Entry<String, OpenStreetMapNode>> iterator = osmNodeTypeMap.entrySet().iterator();
      while(iterator.hasNext()) {
         Map.Entry<String, OpenStreetMapNode> entry = iterator.next();
         String              key = (String) entry.getKey();
         OpenStreetMapNode value = (OpenStreetMapNode) entry.getValue();
         System.out.println( value.toString() );
      }
   }

   // =======================================================================
   // Print contents of TreeMap<String, Way> osmWayTypeMap ...
   // =======================================================================

   public static void printOSMWayTypeMap( TreeMap<String, Way> osmWayTypeMap ) {

      // Header for openstreetmap ways ...

      System.out.println("***");
      System.out.println("*** Summary: TreeMap<String, Way> ... ");
      System.out.println("*** ==================================================== ... ");

      // Summary of categories of openstreetmap way ...

      Iterator<Map.Entry<String, Way>> iterator = osmWayTypeMap.entrySet().iterator();
      while(iterator.hasNext()) {
         Map.Entry<String, Way> entry = iterator.next();
         String key = (String) entry.getKey();
         Way  value = (Way) entry.getValue();
         System.out.println( value.toString() );
      }
   }
  
   // =======================================================================
   // Print contents of TreeMap<String, Relation> osmRelationTypeMap ...
   // =======================================================================

   public static void printOSMRelationTypeMap( TreeMap<String, Relation> osmRelationTypeMap,
                                               TreeMap<String, Way> osmWayTypeMap ) {

      // Header for openstreetmap ways ...

      System.out.println("***");
      System.out.println("*** Summary: TreeMap<String, Relation> ... ");
      System.out.println("*** ==================================================== ... ");

      // Summary of categories of openstreetmap way ...

      Iterator<Map.Entry<String, Relation>> iterator = osmRelationTypeMap.entrySet().iterator();
      while(iterator.hasNext()) {
         Map.Entry<String, Relation> entry = iterator.next();
         String   key = (String) entry.getKey();
         Relation rel = (Relation) entry.getValue();

         StringBuffer buf = new StringBuffer();
         buf.append( String.format("*** Relation[ name = %s ] ... \n", rel.getName() ) );
         buf.append( String.format("               key = %s ... \n", rel.getId() ) );
         buf.append( String.format("              type = %s ... \n", rel.getType() ) );

         List<Member> members = rel.getMembers();
         if( members != null ) {
            int i = 1;
            for (Member member : members) {
                String wayt = member.type;
                String reft = member.ref;

                buf.append( String.format("*** Member [%3d]: type = %s: ref = %s: role = %s ... \n",
                                            i, member.type, member.ref, member.role ) );

                if( wayt.equals("way") == true ) {
                    Way wayref = (Way) osmWayTypeMap.get (reft);
                    if ( wayref == null ) 
                       buf.append( String.format("***             : ERROR -- way not found ... \n" ));
                    else
                       buf.append( String.format("***             : CORRECT -- way found ... \n" ));
                }

                i = i + 1;
            }
         }

         System.out.println( buf.toString() );
      }

   }
  
   // ===========================================================
   // Download Sample OSM file ...
   // ===========================================================

   public static void downloadSample(File file) throws IOException {

      System.out.println("*** In downloadSample(): Downloading OSM data file ...");
      System.out.println("*** ============================================== ...");

      URL downloadUrl = new URL("http://api.openstreetmap.org/api/0.6/map?bbox=11.62973,52.12236,11.63736,52.12853");

      InputStream   in = downloadUrl.openStream();
      OutputStream out = new FileOutputStream(file);
      byte[] buffer = new byte[10000];
      try {
         int len = in.read(buffer);
         while (len > 0) {
            out.write(buffer, 0, len);
            len = in.read(buffer);
         }
      } finally {
         out.close();
         in.close();
      }
   }
}
