/**
 * =====================================================
 * Bounds.java: .....
 *
 * Written by: Mark Austin
 * =====================================================
 */

package osm02.model;

import java.util.Map;

import com.vividsolutions.jts.geom.Coordinate;
import com.vividsolutions.jts.geom.GeometryFactory;
import com.vividsolutions.jts.geom.Point;
import com.vividsolutions.jts.io.WKBWriter;

import org.gavaghan.geodesy.Ellipsoid;
import org.gavaghan.geodesy.GeodeticCalculator;
import org.gavaghan.geodesy.GeodeticCurve;
import org.gavaghan.geodesy.GeodeticMeasurement;
import org.gavaghan.geodesy.GlobalCoordinates;
import org.gavaghan.geodesy.GlobalPosition;

public class Bounds extends AbstractNode {
    public String minlat = "0.0";
    public String minlon = "0.0";
    public String maxlat = "0.0";
    public String maxlon = "0.0";
    public double dMinX  = 0.0;
    public double dMinY  = 0.0;
    public double dMaxX  = 0.0;
    public double dMaxY  = 0.0;
    private GlobalCoordinates base;

    public Bounds() {}

    public Bounds( String minlat, String minlon, String maxlat, String maxlon ) {
        super();
        this.minlat = minlat;
        this.minlon = minlon;
        this.maxlat = maxlat;
        this.maxlon = maxlon;
    }

    // ---------------------------------------------------
    // get bounds on latitude and longitude ....
    // ---------------------------------------------------

    public String getMinLatitude() {
       return minlat;
    }

    public String getMaxLatitude() {
       return maxlat;
    }

    public String getMinLongitude() {
       return minlon;
    }

    public String getMaxLongitude() {
       return maxlon;
    }

    // ---------------------------------------------------
    // get bounds in x-y coordinate system ...
    // ---------------------------------------------------

    public double getMinX() {
       return dMinX;
    }

    public double getMaxX() {
       return dMaxX;
    }

    public double getMinY() {
       return dMinY;
    }

    public double getMaxY() {
       return dMaxY;
    }

    // ----------------------------------------------------------------
    // compute local x- and y- coordinates at (longitude, latitude) ... 
    // ----------------------------------------------------------------

    public double getX( double dLongitude, double dLatitude ) { 
       GeodeticCalculator geoCalc = new GeodeticCalculator();
       Ellipsoid reference = Ellipsoid.WGS84;

       double dMinLongitude = Double.parseDouble( getMinLongitude() );
       GlobalCoordinates temp01 = new GlobalCoordinates( dLatitude, dMinLongitude );
       GlobalCoordinates temp02 = new GlobalCoordinates( dLatitude,    dLongitude );

       GlobalCoordinates temp = new GlobalCoordinates( dLatitude, dLongitude );
       GeodeticCurve geoBase  = geoCalc.calculateGeodeticCurve( reference, temp01, temp02 );
       double xCoord = geoBase.getEllipsoidalDistance();

       // Need to check for points to the left of the bounding box ....

       if( dMinLongitude < dLongitude )
         return  xCoord;
       else
         return -xCoord;
    }

    public double getY( double dLongitude, double dLatitude ) { 
       GeodeticCalculator geoCalc = new GeodeticCalculator();
       Ellipsoid reference = Ellipsoid.WGS84;

       double dMinLatitude      = Double.parseDouble( getMinLatitude() );
       GlobalCoordinates temp01 = new GlobalCoordinates(    dLatitude, dLongitude );
       GlobalCoordinates temp02 = new GlobalCoordinates( dMinLatitude, dLongitude );

       GeodeticCurve geoBase  = geoCalc.calculateGeodeticCurve( reference, temp01, temp02 );
       double yCoord = geoBase.getEllipsoidalDistance();

       // Need to check for points that are below the bounding box ....

       if( dLatitude > dMinLatitude )
          return  yCoord;
       else
          return -yCoord;
    }

    // ---------------------------------------------------
    // compute bounds in x-y coordinate system ...
    // ---------------------------------------------------

    public void computeXYBounds() {
       double dMinLatitude  = Double.parseDouble( getMinLatitude() );
       double dMaxLatitude  = Double.parseDouble( getMaxLatitude() );
       double dMinLongitude = Double.parseDouble( getMinLongitude() );
       double dMaxLongitude = Double.parseDouble( getMaxLongitude() );

       // instantiate the calculator

       GeodeticCalculator geoCalc = new GeodeticCalculator();
       Ellipsoid reference = Ellipsoid.WGS84;

       // Point 01: lower left-hand side ....

       base = new GlobalCoordinates(  dMinLatitude, dMinLongitude );

       // Point 02: lower and upper right-hand side coords ....

       GlobalCoordinates coord02 = new GlobalCoordinates( dMinLatitude, dMaxLongitude );
       GlobalCoordinates coord03 = new GlobalCoordinates( dMaxLatitude, dMaxLongitude );

       // Calculate the 2D geodetic curves

       GeodeticCurve geoBase   = geoCalc.calculateGeodeticCurve(reference,    base, coord02 );
       GeodeticCurve geoHeight = geoCalc.calculateGeodeticCurve(reference, coord02, coord03 );
       double studyWidth       = geoBase.getEllipsoidalDistance();
       double studyHeight      = geoHeight.getEllipsoidalDistance();

       dMaxX = studyWidth;
       dMaxY = studyHeight;
    }
}
