/**
 * =====================================================
 * OpenStreetMapNode.java: .....
 *
 * @author Willy Tiengo
 * =====================================================
 */

package osm02.model;

import com.vividsolutions.jts.geom.Coordinate;
import com.vividsolutions.jts.geom.GeometryFactory;
import com.vividsolutions.jts.geom.Point;
import com.vividsolutions.jts.io.WKBWriter;
import java.util.Map;

import whistle.*;

public class OpenStreetMapNode extends AbstractNode {
    public String      lat;
    public String      lon;
    public Quantity xCoord;
    public Quantity yCoord;

    public OpenStreetMapNode(String id, String visible, String timestamp, String version,
                   String changeset, String user, String uid, String lat,
                   String lon, Map<String, String> tags) {
        super(id, visible, timestamp, version, changeset, user, uid, tags);
        this.lat  = lat;
        this.lon  = lon;

        if ( tags != null )
           this.tags = tags;
    }

    // --------------------------------------------------
    // Retrieve details of the OSM Node .....
    // --------------------------------------------------

    public String getLatitude() {
       return lat;
    }

    public String getLongitude() {
       return lon;
    }

    // --------------------------------------------------
    // Interface for x- and y- coordinates ...
    // --------------------------------------------------

    public void setX ( Quantity x ) {
       this.xCoord = x;
    }

    public Quantity getX () {
       return xCoord;
    }

    public void setY ( Quantity y ) {
       this.yCoord = y;
    }

    public Quantity getY () {
       return yCoord;
    }

    // --------------------------------------------------
    // String representation of OpenStreeMapNode ....
    // --------------------------------------------------

    public String toString() {
       StringBuffer buf = new StringBuffer();

       double dLatitude  = Double.parseDouble(  this.getLatitude() );
       double dLongitude = Double.parseDouble(  this.getLongitude() );

       buf.append( String.format(" Node: [ key = %s] ... \n", getId() ) );
       buf.append( String.format("       [ lon, lat] = [ %f, %f ] ... \n", dLongitude, dLatitude ) );

       if (this.hasName() == true)
       buf.append( String.format("       [ name = %s] ... \n", this.getName() ) );

       if ( this.isAmenity() == true )
          buf.append( String.format("       [ amenity = %s ] ... \n", this.getAmenityType() ) );
       if ( this.isBicycle() == true )
          buf.append( String.format("       [ bicycle = %s ] ... \n", this.getBicycleType() ) );
       if ( this.isBoundary() == true )
          buf.append( String.format("       [ boundary = %s ] ... \n", this.getBoundaryType() ) );
       if ( this.isBuilding() == true )
          buf.append( String.format("       [ building = %s ] ... \n", this.getBuildingType() ) );
       if ( this.isCrossing() == true )
          buf.append( String.format("       [ crossing = %s ] ... \n", this.getCrossingType() ) );
       if ( this.isEntrance() == true )
          buf.append( String.format("       [ entrance = %s ] ... \n", this.getEntranceType() ) );
       if ( this.isHighway() == true )
          buf.append( String.format("       [ highway = %s ] ... \n", this.getHighwayType() ) );
       if ( this.isJunction() == true )
          buf.append( String.format("       [ junction = %s ] ... \n", this.getJunctionType() ) );
       if ( this.isLeisure() == true )
          buf.append( String.format("       [ leisure = %s ] ... \n", this.getLeisureType() ) );
       if ( this.isNatural() == true )
          buf.append( String.format("       [ natural = %s ] ... \n", this.getNaturalType() ) );
       if ( this.isNetwork() == true )
          buf.append( String.format("       [ network = %s ] ... \n", this.getNetworkType() ) );
       if ( this.isPlace() == true )
          buf.append( String.format("       [ place = %s ] ... \n", this.getPlaceType() ) );
       if ( this.isRailway() == true )
          buf.append( String.format("       [ railway = %s ] ... \n", this.getRailwayType() ) );
       if ( this.isRailwayStation() == true )
          buf.append( String.format("       [ railway = %s ] ... \n", this.getRailwayStationType() ) );
       if ( this.isRoute() == true )
          buf.append( String.format("       [ route = %s ] ... \n", this.getRouteType() ) );
       if ( this.isShop() == true )
          buf.append( String.format("       [ shop = %s ] ... \n", this.getShopType() ) );
       if ( this.isSubway() == true )
          buf.append( String.format("       [ subway  = %s ] ... \n", this.getSubwayType() ) );
       if ( this.isTaxiway() == true )
          buf.append( String.format("       [ taxiway = %s ] ... \n", this.getTaxiwayType() ) );
       if ( this.isTourism() == true )
          buf.append( String.format("       [ tourism = %s ] ... \n", this.getTourismType() ) );
       if ( this.isTunnel() == true )
          buf.append( String.format("       [ tunnel = %s ] ... \n", this.getTunnelType() ) );

       return buf.toString();
    }
}
