/*
 * ======================================================================
 * Quantity.java:
 *
 * Written by: Mark Austin
 * ======================================================================
 */

package whistle;

import java.util.*;
import java.lang.Math;

public class Quantity {
   private String     sName;  // quantity name ...
   private String     uName;  // units name ...
   private double     value;  // value ...
   private Dimension dimen;  // dimensions ...
   boolean DEBUG   = false;
   boolean VERBOSE = false;

   private LibraryDimension library = new LibraryDimension();
   private static HashMap d2s = LibraryDimension.getDimenToStringLibrary();
   private static HashMap s2d = LibraryDimension.getStringToLibraryDimen();

   // Constructor method .....

   public Quantity() {
      dimen = new Dimension();
   } 

   public Quantity( String sName ) {
      this.sName = sName;
      this.dimen = new Dimension();
   } 

   public Quantity( double value ) {
      this.value = value;
      this.dimen = new Dimension();
   }

   public Quantity( double value, Dimension dimen ) {
      this.value = value;
      this.dimen = dimen;
      this.uName = (String) d2s.get((Dimension) this.dimen );
   }

   public Quantity( String sName, double value, Dimension dimen ) {
      this.sName = sName;
      this.value = value;
      this.dimen = dimen;
      this.uName = (String) d2s.get((Dimension) this.dimen );
   }

   // =================================================
   // Set/get quantity attributes ....
   // =================================================

   public void setName ( String sName ) {
      this.sName = sName;
   }

   public String getName () {
      return sName;
   }

   public void setUnitsName () {
      this.uName = (String) d2s.get((Dimension) this.dimen );
   }

   public void setUnitsName ( String uName ) {
      this.uName = uName;
   }

   public String getUnitsName () {
      return uName;
   }

   public void setQuantity ( Quantity value ) {
      this.value = value.getValue();
      this.dimen = (Dimension) s2d.get( value.getUnitsName() );
      this.uName = value.getUnitsName();
   }

   public void setValue ( double value ) {
      this.value = value;
   }

   public double getValue () {
      return value;
   }

   public void setDimension ( Dimension dimen ) {
      this.dimen = dimen;
      this.uName = (String) d2s.get((Dimension) this.dimen );
   }

   public Dimension getDimension () {
      return dimen;
   }

   // =================================================
   // Add quantities ....
   // =================================================

   public Quantity add( Quantity q02 ) {
      Quantity result = new Quantity();
      boolean DEBUG = true;

      // Check that dimensions are compatible ...

      Dimension d01 = this.getDimension();
      Dimension d02 =  q02.getDimension();

      // Apply conversion for compatible units ....

      if(d01.compatibleUnits(d02) == true ) {
         q02 = q02.convert( d01 );
      } else { 
         System.out.println("*** In src/whistle/Quantity.add() ....");
         System.out.println("*** ERROR: Units are incompatible ....");
         return null;
      }

      // Add quantities ....

      String uName = (String) d2s.get((Dimension) this.dimen );
      if(d01.sameDimension(d02) == true ) {
         result.value = this.value + q02.getValue();
         result.dimen = this.dimen;
         result.setUnitsName  ( uName );
      } else {
         double sf01 = this.dimen.getScaleFactor();
         double sf02 =  q02.dimen.getScaleFactor();
         result.value = this.value + (sf02/sf01)*q02.getValue();
         result.dimen = this.dimen;
         result.setUnitsName  ( uName );
      }

      return result;
   }

   // =================================================
   // Subtract quantities ....
   // =================================================

   public Quantity sub( Quantity q02 ) {
      Quantity result = new Quantity();

      // Check that dimensions are compatible ...

      Dimension d01 = this.getDimension();
      Dimension d02 =  q02.getDimension();

      // Apply conversion for compatible units ....

      if(d01.compatibleUnits(d02) == true ) {
         q02 = q02.convert( d01 );
      } else { 
         System.out.println("*** In src/whistle/Quantity.sub() ....");
         System.out.println("*** ERROR: Units are incompatible ....");
         return null;
      }

      // Apply conversion for compatible units ....

      if(d01.compatibleUnits(d02) == true ) {
         q02 = q02.convert( d01 );
      } 

      // Subtract quantities ....

      String uName = (String) d2s.get((Dimension) this.dimen );
      if(d01.sameDimension(d02) == true ) {
         result.value = this.value - q02.getValue();
         result.dimen = this.dimen;
         result.setUnitsName  ( uName );
      } else {
         double sf01 = this.dimen.getScaleFactor();
         double sf02 =  q02.dimen.getScaleFactor();
         result.value = this.value - (sf02/sf01)*q02.getValue();
         result.dimen = this.dimen;
         result.setUnitsName  ( uName );
      }

      return result;
   }

   // =================================================
   // Multiply quantities ....
   // =================================================

   public Quantity mult( Quantity q02 ) {
      Quantity result = new Quantity();

      Dimension      d01   = this.getDimension();
      Dimension      d02   =  q02.getDimension();
      DimensionType dt01   =  d01.getUnitsType();
      DimensionType dt02   =  d02.getUnitsType();
      double scaleFactor01 = d01.getScaleFactor();
      double scaleFactor02 = d02.getScaleFactor();

      if ( DEBUG == true ) {
         System.out.println ("*** Enter Quantity.mult() ... ");
         System.out.println ("*** dt01 = " + d01.toString());
         System.out.println ("*** dt02 = " + d02.toString());
      }

      // Make units conversion ....

      if ( dt01 != dt02 ) {
         switch ( dt02 ) {
             case US:
                  q02 = q02.convert( DimensionType.SI );
                  break;
             case SI:
                  q02 = q02.convert( DimensionType.US );
                  break;
             default:
                  break;
         }

         d02  = q02.getDimension();
         dt02 = d02.getUnitsType();
         scaleFactor02 = d02.getScaleFactor();
      }

      // Simply multiply quantities with identical units types....

      Dimension d03 = d01.add(d02);
      d03.setScaleFactor ( 1.0 );
      d03.setUnitsType ( dt01 );
      String uName03 = (String) d2s.get( d03 );
      result.setDimension ( d03 );
      result.setUnitsName ( uName03 );
      result.setValue ( this.getValue() * scaleFactor01 * scaleFactor02 * q02.getValue() );

      return result;
   }

   // =================================================
   // Divide quantities ....
   // =================================================

   public Quantity div( Quantity q02 ) {
      Quantity result = new Quantity();

      Dimension      d01   = this.getDimension();
      Dimension      d02   =  q02.getDimension();
      DimensionType dt01   =  d01.getUnitsType();
      DimensionType dt02   =  d02.getUnitsType();
      double scaleFactor01 = d01.getScaleFactor();
      double scaleFactor02 = d02.getScaleFactor();

      // Make units coversion ....

      if ( dt01 != dt02 ) {
         switch ( dt02 ) {
             case US:
                  q02 = q02.convert( DimensionType.SI );
                  break;
             case SI:
                  q02 = q02.convert( DimensionType.US );
                  break;
             default:
                  break;
         }

         d02  = q02.getDimension();
         dt02 = d02.getUnitsType();
         scaleFactor02 = d02.getScaleFactor();

         if ( DEBUG == true ) {
            System.out.println("*** Part 01/2: Converted q02 = " + q02 );
            System.out.println("*** ScaleFactor01 = " + scaleFactor01 );
            System.out.println("*** ScaleFactor02 = " + scaleFactor02 );
         }
      }

      // Simply divide quantities with identical units types....

      Dimension d03 = d01.sub(d02);
      d03.setScaleFactor ( 1.0 );
      d03.setUnitsType ( dt01 );

      String uName03 = (String) d2s.get( d03 );
      result.setDimension ( d03 );
      result.setUnitsName ( uName03 );
      result.setValue ( (this.getValue() * scaleFactor01) /
                        (scaleFactor02 * q02.getValue()) );

      return result;
   }

   // =================================================
   // Pow quantities ....
   // =================================================

   public Quantity pow( Quantity q02 ) {
      Quantity result = new Quantity();

      // Scale dimensions by an integer power ....

      Dimension d01 = this.getDimension();
      int n = (int) q02.getValue();

      result.setDimension ( d01.scale(n) );
      String uName = (String) d2s.get( d01.scale(n) );

      result.setUnitsName  ( uName );
      result.setValue ( Math.pow ( this.getValue(), (double ) n) );

      return result;
   }

   // =========================================================
   // Builtin function abs() ...
   // =========================================================

   public Quantity abs() {
      Quantity result = new Quantity();

      Dimension d01  = this.getDimension();
      double value01 = this.value;

      result.setValue ( Math.abs( value01 ) );
      result.setDimension ( d01 );

      return result;
   }

   // =========================================================
   // Builtin triganometric functions ...
   // =========================================================

   public Quantity sin() {
      Quantity result = new Quantity();

      Dimension d01  = this.getDimension();
      double value01 = this.value;

      result.setValue ( Math.sin( value01 ) );
      return result;
   }

   public Quantity cos() {
      Quantity result = new Quantity();

      Dimension d01  = this.getDimension();
      double value01 = this.value;

      result.setValue ( Math.cos( value01 ) );
      return result;
   }

   // =========================================================
   // Builtin function random() ...
   // =========================================================

   public Quantity random() {
      Quantity result = new Quantity();

      Dimension d01  = this.getDimension();
      double value01 = this.value;

      result.setValue ( Math.random() );
      return result;
   }

   // =========================================================
   // Builtin function sqrt() ...
   // =========================================================

   public Quantity sqrt() {
      Quantity result = new Quantity();

      Dimension d01  = this.getDimension();
      double value01 = this.value;

      result.setValue ( Math.sqrt( value01 ) );
      result.setDimension ( d01.scale (0.5) );

      return result;
   }

   // =========================================================
   // Builtin function max() ...
   // =========================================================

   public Quantity max( Quantity q01 ) {
      Quantity result = new Quantity();

      // Only proceed if the units are compatible ....

      Dimension d01 = this.getDimension();
      Dimension d02 =  q01.getDimension();

      if (d01.sameUnits(d02) != true ) {
         System.out.println("*** ERROR in whistle/Quantity.max() ... ");
         System.out.println("*** Incompatible units ... ");
         System.out.println("*** " + this.toString() );
         System.out.println("*** " +  q01.toString() );
         return result;
      }

      double value01 = this.value;
      double scale01 = d01.getScaleFactor();
      double value02 = q01.getValue();
      double scale02 = d02.getScaleFactor();

      if ( value01*scale01 >= value02*scale02 ) {
         result.setValue ( value01 );
         result.setDimension ( d01 );
      } else {
         result.setValue ( value02 );
         result.setDimension ( d02 );
      }

      return result;
   }

   // =========================================================
   // Builtin function min() ...
   // =========================================================

   public Quantity min( Quantity q01 ) {
      Quantity result = new Quantity();

      // Only proceed if the units are compatible ....

      Dimension d01 = this.getDimension();
      Dimension d02 =  q01.getDimension();

      if (d01.sameUnits(d02) != true ) {
         System.out.println("*** ERROR in whistle/Quantity.min() ... ");
         System.out.println("*** Incompatible units ... ");
         System.out.println("*** " + this.toString() );
         System.out.println("*** " +  q01.toString() );
         return result;
      }

      double value01 = this.value;
      double scale01 = d01.getScaleFactor();
      double value02 = q01.getValue();
      double scale02 = d02.getScaleFactor();

      if ( value01*scale01 <= value02*scale02 ) {
         result.setValue ( value01 );
         result.setDimension ( d01 );
      } else {
         result.setValue ( value02 );
         result.setDimension ( d02 );
      }

      return result;
   }

   // =========================================================
   // Relational expressions between quantities ...
   // =========================================================

   // Evaluate "greater than" relational expression ...

   public boolean gt( Quantity q02 ) {
      boolean result = false;

      // Check that dimensions are compatible ...

      Dimension d01 = this.getDimension();
      Dimension d02 =  q02.getDimension();

      // Apply conversion for compatible units ....

      if(d01.compatibleUnits(d02) == true ) {
         q02 = q02.convert( d01 );
      } else { 
         System.out.println("*** In src/whistle/Quantity.gt() ....");
         System.out.println("*** ERROR: Units are incompatible ....");
         return result;
      }

      // Add quantities ....

      if(d01.sameDimension(d02) == true ) {
         result = this.value > q02.getValue();
      } else {
         double sf01 = this.dimen.getScaleFactor();
         double sf02 =  q02.dimen.getScaleFactor();
         result = this.value > (sf02/sf01)*q02.getValue();
      }

      return result;
   }

   // Evaluate "greater than or equal to" relational expression ...

   public boolean ge( Quantity q02 ) {
      boolean result = false;

      // Check that dimensions are compatible ...

      Dimension d01 = this.getDimension();
      Dimension d02 =  q02.getDimension();

      // Apply conversion for compatible units ....

      if(d01.compatibleUnits(d02) == true ) {
         q02 = q02.convert( d01 );
      } else { 
         System.out.println("*** In src/whistle/Quantity.ge() ....");
         System.out.println("*** ERROR: Units are incompatible ....");
         return result;
      }

      // Add quantities ....

      if(d01.sameDimension(d02) == true ) {
         result = (this.value >= q02.getValue());
      } else {
         double sf01 = this.dimen.getScaleFactor();
         double sf02 =  q02.dimen.getScaleFactor();
         result = (this.value >= (sf02/sf01)*q02.getValue());
      }

      return result;
   }

   // Evaluate "less than" relational expression ...

   public boolean lt( Quantity q02 ) {
      boolean result = false;

      // Check that dimensions are compatible ...

      Dimension d01 = this.getDimension();
      Dimension d02 =  q02.getDimension();

      // Apply conversion for compatible units ....

      if(d01.compatibleUnits(d02) == true ) {
         q02 = q02.convert( d01 );
      } else { 
         System.out.println("*** In src/whistle/Quantity.lt() ....");
         System.out.println("*** ERROR: Units are incompatible ....");
         return result;
      }

      // Add quantities ....

      if(d01.sameDimension(d02) == true ) {
         result = (this.value < q02.getValue());
      } else {
         double sf01 = this.dimen.getScaleFactor();
         double sf02 =  q02.dimen.getScaleFactor();
         result = (this.value < (sf02/sf01)*q02.getValue());
      }

      return result;
   }

   // Evaluate "less than or equal to" relational expression ...

   public boolean le( Quantity q02 ) {
      boolean result = false;

      // Check that dimensions are compatible ...

      Dimension d01 = this.getDimension();
      Dimension d02 =  q02.getDimension();

      // Apply conversion for compatible units ....

      if(d01.compatibleUnits(d02) == true ) {
         q02 = q02.convert( d01 );
      } else { 
         System.out.println("*** In src/whistle/Quantity.le() ....");
         System.out.println("*** ERROR: Units are incompatible ....");
         return result;
      }

      // Add quantities ....

      if(d01.sameDimension(d02) == true ) {
         result = (this.value <= q02.getValue());
      } else {
         double sf01 = this.dimen.getScaleFactor();
         double sf02 =  q02.dimen.getScaleFactor();
         result = (this.value <= (sf02/sf01)*q02.getValue());
      }

      return result;
   }

   // Evaluate "equal to" relational expression ...

   public boolean eq( Quantity q02 ) {
      boolean result = false;

      // Check that dimensions are compatible ...

      Dimension d01 = this.getDimension();
      Dimension d02 =  q02.getDimension();

      // Apply conversion for compatible units ....

      if(d01.compatibleUnits(d02) == true ) {
         q02 = q02.convert( d01 );
      } else { 
         System.out.println("*** In src/whistle/Quantity.eq() ....");
         System.out.println("*** ERROR: Units are incompatible ....");
         return result;
      }

      // Add quantities ....

      if(d01.sameDimension(d02) == true ) {
         result = (this.value == q02.getValue());
      } else {
         double sf01 = this.dimen.getScaleFactor();
         double sf02 =  q02.dimen.getScaleFactor();
         result = (this.value == (sf02/sf01)*q02.getValue());
      }

      return result;
   }

   // Evaluate "not equal to" relational expression ...

   public boolean ne( Quantity q02 ) {
      boolean result = true;

      // Check that dimensions are compatible ...

      Dimension d01 = this.getDimension();
      Dimension d02 =  q02.getDimension();

      // Apply conversion for compatible units ....

      if(d01.compatibleUnits(d02) == true ) {
         q02 = q02.convert( d01 );
      } else { 
         System.out.println("*** In src/whistle/Quantity.neq() ....");
         System.out.println("*** Note: Units are incompatible ....");
         return result;
      }

      // Add quantities ....

      if(d01.sameDimension(d02) == true ) {
         result = (this.value != q02.getValue());
      } else {
         double sf01 = this.dimen.getScaleFactor();
         double sf02 =  q02.dimen.getScaleFactor();
         result = (this.value != (sf02/sf01)*q02.getValue());
      }

      return result;
   }

   // =========================================================
   // Relational expressions between quantities ...
   // =========================================================

   // Convert units: string argument ....

   public Quantity convert( String sName ) {
      Quantity result = new Quantity();

      // Check that string name is a well defined unit ....

      Dimension d01 = (Dimension) s2d.get(sName);
      if ( d01 == null ) {
         System.out.println("*** In src/whistle/Quantity.convert() ... ");
         System.out.printf ("*** ERROR: Units %s are not defined ... \n", sName );
         return result;
      }

      // Compute new value .....

      return convert(d01);
   }

   // ---------------------------------------------------
   // Convert units: dimensiontype argument ....
   // ---------------------------------------------------

   public Quantity convert( DimensionType dtype ) {
      Quantity result      = new Quantity();
      Dimension     dimen  =  this.getDimension();
      DimensionType dt01   = dimen.getUnitsType();
      double scaleFactor01 = dimen.getScaleFactor();
      boolean DEBUG = false;

      // Get current exponent values ...

      double lengthExponent = dimen.getLengthExponent();
      double massExponent   = dimen.getMassExponent();
      double timeExponent   = dimen.getTimeExponent();
      double tempExponent   = dimen.getTempExponent();
      double radianExponent = dimen.getRadianExponent();

      // Compute SI to US conversion factor ....

      double conversionFactor = 1.0;
      if ( dt01 == DimensionType.SI && dtype == DimensionType.US ) {

         double factorLength = 1.0;
         if ( lengthExponent != 0 )
            factorLength = Math.pow (  (1.0/0.0254), lengthExponent );

         double factorMass = 1.0;
         if ( massExponent != 0 )
            factorMass = Math.pow ( (1.0/0.4535924), massExponent );

         conversionFactor = factorLength*factorMass;
      }

      // Compute US to SI conversion factor ....

      if ( dt01 == DimensionType.US && dtype == DimensionType.SI ) {

         double factorLength = 1.0;
         if ( lengthExponent != 0 )
            factorLength = Math.pow (  0.0254, lengthExponent );

         double factorMass = 1.0;
         if ( massExponent != 0 )
            factorMass = Math.pow ( 0.4535924, massExponent );

         conversionFactor = factorLength*factorMass;
      }

      // Create new dimension ...

      Dimension dnew = new Dimension();
      dnew.setLengthExponent( lengthExponent );
      dnew.setMassExponent(   massExponent   );
      dnew.setTimeExponent(   timeExponent   );
      dnew.setTempExponent(   tempExponent   );
      dnew.setRadianExponent( radianExponent );
      dnew.setScaleFactor( 1.0 );

      if ( dtype == DimensionType.SI ) {
         dnew.setUnitsType( DimensionType.SI );
      } else
         dnew.setUnitsType( DimensionType.US );

      // Compute required quantity ...

      String uName = (String) d2s.get( dnew );
      double value = conversionFactor*scaleFactor01*(this.value);

      result.setDimension ( dnew );
      result.setValue( value );
      result.setUnitsName ( uName );

      if ( DEBUG == true ) {
         System.out.println("*** result = " + result.toString() );
         System.out.println("*** Leaving Quality.convert() ....   ");
      }

      return result;
   }

   // ---------------------------------------------------
   // Convert units: dimension argument ....
   // ---------------------------------------------------

   public Quantity convert( Dimension d01 ) {
      Quantity result = new Quantity();

      // Check compatibility of current and required dimensions ...

      if ( this.getDimension().compatibleUnits(d01) == false ) {
         System.out.println("*** In src/whistle/Quantity.convert() ... ");
         System.out.println("*** ERROR: Current units are incompatible with required units ... ");
         return result;
      }

      // Current and required units are of the same type ....

      DimensionType dt01 = dimen.getUnitsType();
      DimensionType dt02 =   d01.getUnitsType();

      if ( dt01 == dt02 ) {
         double scaleFactor01 = dimen.getScaleFactor();
         double scaleFactor02 =   d01.getScaleFactor();
         double value = (scaleFactor01/scaleFactor02)*(this.value);
         result.setDimension ( d01 );
         result.setValue( value );
         return result;
      } 

      // Compute SI to US conversion ....

      double conversionFactor = 1.0;
      if ( dt01 == DimensionType.SI && dt02 == DimensionType.US ) {
         double lengthExponent = dimen.getLengthExponent();
         double massExponent   = dimen.getMassExponent();
         double timeExponent   = dimen.getTimeExponent();
         double tempExponent   = dimen.getTempExponent();
         double radianExponent = dimen.getRadianExponent();

         // Compute units conversion factor ....

         double factorLength = 1.0;
         if ( lengthExponent != 0 )
            factorLength = Math.pow (  (1.0/0.0254), lengthExponent );

         double factorMass = 1.0;
         if ( massExponent != 0 )
            factorMass = Math.pow ( (1.0/0.4535924), massExponent );

         conversionFactor = factorLength*factorMass;
      }

      // Compute US to SI conversion ....

      if ( dt01 == DimensionType.US && dt02 == DimensionType.SI ) {
         double lengthExponent = dimen.getLengthExponent();
         double massExponent   = dimen.getMassExponent();
         double timeExponent   = dimen.getTimeExponent();
         double tempExponent   = dimen.getTempExponent();
         double radianExponent = dimen.getRadianExponent();

         // Compute units conversion factor ....

         double factorLength = 1.0;
         if ( lengthExponent != 0 )
            factorLength = Math.pow (  0.0254, lengthExponent );

         double factorMass = 1.0;
         if ( massExponent != 0 )
            factorMass = Math.pow ( 0.4535924, massExponent );

         conversionFactor = factorLength*factorMass;
      }

      // Compute required quantity ...

      double scaleFactor01 = dimen.getScaleFactor();
      double scaleFactor02 =   d01.getScaleFactor();
      double value = conversionFactor*(scaleFactor01/scaleFactor02)*(this.value);
      result.setDimension ( d01 );
      result.setValue( value );

      return result;
   }

   // =========================================================
   // Create clone (i.e., deep copy) of a Quantity object ...
   // =========================================================

   public Object clone() {
      try {
         Quantity q = (Quantity) super.clone();
         q.dimen = (Dimension) this.dimen.clone();
         return q;
      } catch (CloneNotSupportedException e) {
         return null;
      }
   }

   // =========================================================
   // String representation of quantities ...
   // =========================================================

   public String toString() {
      StringBuffer buf = new StringBuffer();

      if ( VERBOSE == true ) {
         buf.append("Quantity [ " + this.sName + " ] = [" + this.value + ", " + this.uName + "]\n");
         buf.append( dimen.toString() );
      } else {
         buf.append( String.format("[ %.4g, %s]", this.value, this.uName ) );
      }

      return buf.toString();
   }

   // =========================================================
   // Exercise methods in Quantity ....
   // =========================================================

   public static void main( String args [] ) {

      System.out.println("");
      System.out.println("*** Exercise methods in whistle/Quantity ...");
      System.out.println("*** ========================================== ");

      Quantity q01 = new Quantity();
      System.out.println ("Quantity q01 ... " );
      System.out.println ( q01.toString() );

   }
}
